# -*- coding: utf-8 -*-
# @package Download Image Sentinel
"""
Install:

#        $ sudo pip3 install sentinelsat
	$ sudo apt-get install libgdal-dev

Example:

        $ python3 -m doctest -v APISentinel.py
"""

import os
from oauthlib.oauth2 import BackendApplicationClient
from requests_oauthlib import OAuth2Session
#from sentinelsat.sentinel import SentinelAPI
from datetime import date
from collections import OrderedDict
import time
import shapely.wkt
import shapely.geometry
import json
#from osgeo import ogr

def sentinelhub_compliance_hook(response):
	response.raise_for_status()
	return response
class APISentinel(object):
	""" Class for Sentinel satellites configuration
	Test Case
	>>> sentinel = APISentinel("cfg.json")
	>>> products = sentinel.getProducts("POLYGON((-89.99450683593753 21.279137394108716,-89.13757324218751 21.2996106049456,-89.30236816406251 20.68418377935238,-90.0494384765625 20.715015145512098,-89.99450683593753 21.279137394108716))", ('20151219', date(2015,12,29)), {"platformname":"Sentinel-2"})
	>>> print(len(products))
	6

	"""

	def __init__(self, client_id, client_secret, s3cfg):
		"""The constructor Initialize Sentinel Data.

		Args:
		    self: The object pointer.
		    cfg: JSON configuration file.

		Returns:
		    pointer: The object pointer.
		"""
		self.client_id = client_id
		self.client_secret = client_secret
		self.s3cfg=s3cfg
		self.oauth_token_url='https://identity.dataspace.copernicus.eu/auth/realms/CDSE/protocol/openid-connect/token'
		
		# Create a session
		client = BackendApplicationClient(client_id=client_id)
		self.oauth = OAuth2Session(client=client)
		self.oauth.register_compliance_hook("access_token_response", sentinelhub_compliance_hook)
		# Get token for the session
		self.token = self.oauth.fetch_token(token_url=self.oauth_token_url, client_secret=client_secret)


	def getProducts(self, area, date, searchParameters):
		"""Gets Sentinel products list that match the search parameters.

		Args:
			self (pointer): The object pointer.
			area (str): Area of interest using a Polygon in WKT format.
			date (tuple): A tuple contining the initial and end dates. 
			searchParameters (dict): Additional keywords can be used to specify other query parameters, 

		Example:
			getProducts("POLYGON((-89.99 21.27,-89.13 21.29,-89.30 20.68,-90.04 20.71,-89.99 21.27))", ('20150101', '20181024'), {"platformname":"Sentinel-2", "cloudcoverpercentage":"[0 TO 40]"})

		Returns:
			OrderedDict: Sentinel Product list found.
		"""
		url = "https://sh.dataspace.copernicus.eu/api/v1/catalog/1.0.0/search"

		data = {
			"intersects": shapely.geometry.mapping(shapely.wkt.loads(area)),
			"datetime": date[0].strftime('%Y-%m-%dT%H:%M:%SZ')+"/"+date[1].strftime('%Y-%m-%dT%H:%M:%SZ'),
			"collections": ["sentinel-2-l2a"],
			"limit": 50,
		}
		data.update(searchParameters)
		productList = {}
		response = None
		while  response is None or response.status_code==200:
			response = self.oauth.post(url, json=data)
			content = json.loads(response.content)
			for i in json.loads(response.content)["features"]:
				productList[i["id"]]={"filename":i["assets"]["data"]["href"]}
			if "next" in content["context"]:
				data["next"]= content["context"]["next"]
				print(content["context"])
			else:
				break;

		return productList

	def downloadProducts(self, products, outdir):
		"""Download product(s) list Sentinel

			Args:
				self (pointer): The object pointer.
				products (OrderedDict): Product(s) to download.
				dir (str): destination directory.
		"""
		os.chdir(outdir)
		#        self.api.download_all(products)
		longTermArchiveList=[]
		for p in products:
			print(products[p]['filename'])
			try:
#				info = self.api.download(p)
				os.system("s3cmd -c "+self.s3cfg+" -r get "+products[p]['filename'])
				base_id=p.replace(".SAFE","")
				os.system("zip -r "+base_id+".zip "+p+"/")
				os.system("rm -r "+p)
#				rm -r $DIRNAME/
#				if not info["Online"]:
#					print("Adding producto to long term list")
#					longTermArchiveList.append(p)
				time.sleep(1)
			except:	

				print ("ERROR")

				
		return longTermArchiveList

#		def filterProducts(self, productList):
#		#, user_footprint, threshold):

#			products = productList.copy()
#			for p in productList:
#				if productList[p]['filename'].find("OPER_PRD") != -1:
#					del products[p]


#			return products


